(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('crypto')) :
    typeof define === 'function' && define.amd ? define(['exports', 'crypto'], factory) :
    (factory((global.actrix = {}),global.crypto));
}(this, (function (exports,crypto) { 'use strict';

    crypto = crypto && crypto.hasOwnProperty('default') ? crypto['default'] : crypto;

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    function __awaiter(thisArg, _arguments, P, generator) {
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f) throw new TypeError("Generator is already executing.");
            while (_) try {
                if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
                if (y = 0, t) op = [0, t.value];
                switch (op[0]) {
                    case 0: case 1: t = op; break;
                    case 4: _.label++; return { value: op[1], done: false };
                    case 5: _.label++; y = op[1]; op = [0]; continue;
                    case 7: op = _.ops.pop(); _.trys.pop(); continue;
                    default:
                        if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                        if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                        if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                        if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                        if (t[2]) _.ops.pop();
                        _.trys.pop(); continue;
                }
                op = body.call(thisArg, _);
            } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
            if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
        }
    }

    function createProxy(actorSystem, targetAddressorActorRef, sender, ask) {
        if (ask === void 0) { ask = false; }
        return new Proxy({}, {
            get: function (target, prop, receiver) {
                return function () {
                    var payload = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        payload[_i] = arguments[_i];
                    }
                    return ask
                        ? actorSystem.sendMessageAndWait.apply(actorSystem, [targetAddressorActorRef,
                            prop,
                            sender || null].concat(payload)) : actorSystem.sendMessage.apply(actorSystem, [targetAddressorActorRef,
                        prop,
                        sender || null].concat(payload));
                };
            }
        });
    }
    var ActorRef = /** @class */ (function () {
        function ActorRef(address, actorSystem) {
            this.address = address;
            this.actorSystem = actorSystem;
        }
        ActorRef.prototype.send = function (sender) {
            return createProxy(this.actorSystem, this.address, sender);
        };
        ActorRef.prototype.ask = function (sender) {
            return createProxy(this.actorSystem, this.address, sender, true);
        };
        return ActorRef;
    }());
    var Actor = /** @class */ (function () {
        function Actor(name, address, actorSystem, options, strategies) {
            var _this = this;
            this.address = address;
            this.actorSystem = actorSystem;
            this.strategies = strategies;
            this.mailBox = [];
            this.context = {
                senderAddress: null,
                senderRef: null
            };
            this.pushToMailbox = function (type, senderAddress) {
                var payload = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    payload[_i - 2] = arguments[_i];
                }
                _this.log("A new message with type " + type + " and payload " + payload + " from sender " + senderAddress + " is received");
                if (_this.strategies && _this.strategies.includes("IgnoreOlderMessageWithTheSameType")) {
                    if (_this.currentlyProcessedMessage && _this.currentlyProcessedMessage.type === type) {
                        _this.cancelCurrentExecution();
                    }
                }
                try {
                    _this.onNewMessage.apply(_this, [type, senderAddress].concat(payload));
                }
                catch (error) {
                    _this.log("Caught an exception on the implementation of 'onNewMessage' on actor " + _this.name, error);
                }
                var promise = new Promise(function (resolve, reject) {
                    _this.mailBox.push({
                        type: type,
                        payload: payload,
                        senderAddress: senderAddress,
                        callback: function (error, result) {
                            if (error) {
                                reject(error);
                            }
                            else {
                                resolve(result);
                            }
                        }
                    });
                });
                _this.scheduleNextTick();
                return promise;
            };
            this.ref = function (address) {
                return _this.actorSystem.ref(address);
            };
            this.onNewMessage = function (type, senderAddress) {
                var payload = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    payload[_i - 2] = arguments[_i];
                }
                // should be overridden by implementator (when necessary)
            };
            this.cancelCurrentExecution = function () {
                _this.log("Cancel current execution");
                if (_this.currentPromise && typeof _this.currentPromise.cancel === "function") {
                    _this.currentPromise.cancel();
                }
            };
            this.scheduleNextTick = function () {
                if (!_this.timerId) {
                    _this.timerId = setTimeout(_this.executeTick);
                }
            };
            this.executeTick = function () { return __awaiter(_this, void 0, void 0, function () {
                var mail, success, result, _a, type, payload, senderAddress, callback, error_1;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            mail = this.mailBox.shift();
                            success = false;
                            _a = mail, type = _a.type, payload = _a.payload, senderAddress = _a.senderAddress, callback = _a.callback;
                            this.context = {
                                senderAddress: senderAddress,
                                senderRef: senderAddress ? this.ref(senderAddress) : null
                            };
                            if (this.strategies && this.strategies.includes("IgnoreOlderMessageWithTheSameType")) {
                                if (this.mailBox.find(function (message) { return message.type === type; })) {
                                    this.log("Ignoring message with type " + type + " because there are more messages with the same type in the queue.");
                                    result = "There are new messages with the same type. This message is ignored";
                                }
                            }
                            this.currentlyProcessedMessage = mail;
                            if (!!result) return [3 /*break*/, 4];
                            this.currentPromise = this.handleMessage.apply(this, [type].concat(payload));
                            _b.label = 1;
                        case 1:
                            _b.trys.push([1, 3, , 4]);
                            return [4 /*yield*/, this.currentPromise];
                        case 2:
                            result = _b.sent();
                            this.log("Output of the handled message", result);
                            success = true;
                            return [3 /*break*/, 4];
                        case 3:
                            error_1 = _b.sent();
                            this.log("Caught an exception when handling message", error_1);
                            result = error_1;
                            return [3 /*break*/, 4];
                        case 4:
                            this.currentlyProcessedMessage = undefined;
                            this.currentPromise = undefined;
                            callback(success ? undefined : result, success ? result : undefined);
                            if (this.timerId) {
                                clearTimeout(this.timerId);
                                this.timerId = null;
                            }
                            if (this.mailBox.length) {
                                this.scheduleNextTick();
                            }
                            return [2 /*return*/];
                    }
                });
            }); };
            this.name = name;
            this.timerId = null;
            setTimeout(function () {
                _this.init(options);
            });
        }
        Actor.prototype.sendTo = function (targetRef) {
            return createProxy(this.actorSystem, targetRef, this.address);
        };
        Actor.prototype.askTo = function (targetRef) {
            return createProxy(this.actorSystem, targetRef, this.address, true);
        };
        // For some reason the typings is not working properly
        Actor.prototype.sendToSelf = function () {
            // return this.at<ValidActorMethodProps<this>>(this.address);
            return this.sendTo(this.address); // TODO: introduce generic for actor
        };
        Actor.prototype.init = function (options) {
            // can be implemented by the concrete actor
        };
        Actor.prototype.log = function () {
            var message = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                message[_i] = arguments[_i];
            }
            if ((process.env && process.env.ACTRIX_DEBUG) ||
                (typeof window !== "undefined" && window.ACTRIX_DEBUG)) {
                console.log.apply(console, ["[" + this.name + "]"].concat(message));
            }
        };
        // TODO: K extends keyof this is not actually the proper solution,
        // good for now though since it doesn't affect end-user
        Actor.prototype.handleMessage = function (type) {
            var payload = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                payload[_i - 1] = arguments[_i];
            }
            this.log("Handling message of type " + type + " and payload " + payload);
            return (_a = this)[type].apply(_a, payload);
            var _a;
        };
        return Actor;
    }());

    // Unique ID creation requires a high quality random # generator.  In node.js
    // this is pretty straight-forward - we use the crypto API.



    var rng = function nodeRNG() {
      return crypto.randomBytes(16);
    };

    /**
     * Convert array of 16 byte values to UUID string format of the form:
     * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
     */
    var byteToHex = [];
    for (var i = 0; i < 256; ++i) {
      byteToHex[i] = (i + 0x100).toString(16).substr(1);
    }

    function bytesToUuid(buf, offset) {
      var i = offset || 0;
      var bth = byteToHex;
      return bth[buf[i++]] + bth[buf[i++]] +
              bth[buf[i++]] + bth[buf[i++]] + '-' +
              bth[buf[i++]] + bth[buf[i++]] + '-' +
              bth[buf[i++]] + bth[buf[i++]] + '-' +
              bth[buf[i++]] + bth[buf[i++]] + '-' +
              bth[buf[i++]] + bth[buf[i++]] +
              bth[buf[i++]] + bth[buf[i++]] +
              bth[buf[i++]] + bth[buf[i++]];
    }

    var bytesToUuid_1 = bytesToUuid;

    // **`v1()` - Generate time-based UUID**
    //
    // Inspired by https://github.com/LiosK/UUID.js
    // and http://docs.python.org/library/uuid.html

    var _nodeId;
    var _clockseq;

    // Previous uuid creation time
    var _lastMSecs = 0;
    var _lastNSecs = 0;

    // See https://github.com/broofa/node-uuid for API details
    function v1(options, buf, offset) {
      var i = buf && offset || 0;
      var b = buf || [];

      options = options || {};
      var node = options.node || _nodeId;
      var clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq;

      // node and clockseq need to be initialized to random values if they're not
      // specified.  We do this lazily to minimize issues related to insufficient
      // system entropy.  See #189
      if (node == null || clockseq == null) {
        var seedBytes = rng();
        if (node == null) {
          // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
          node = _nodeId = [
            seedBytes[0] | 0x01,
            seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]
          ];
        }
        if (clockseq == null) {
          // Per 4.2.2, randomize (14 bit) clockseq
          clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
        }
      }

      // UUID timestamps are 100 nano-second units since the Gregorian epoch,
      // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
      // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
      // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.
      var msecs = options.msecs !== undefined ? options.msecs : new Date().getTime();

      // Per 4.2.1.2, use count of uuid's generated during the current clock
      // cycle to simulate higher resolution clock
      var nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1;

      // Time since last uuid creation (in msecs)
      var dt = (msecs - _lastMSecs) + (nsecs - _lastNSecs)/10000;

      // Per 4.2.1.2, Bump clockseq on clock regression
      if (dt < 0 && options.clockseq === undefined) {
        clockseq = clockseq + 1 & 0x3fff;
      }

      // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
      // time interval
      if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
        nsecs = 0;
      }

      // Per 4.2.1.2 Throw error if too many uuids are requested
      if (nsecs >= 10000) {
        throw new Error('uuid.v1(): Can\'t create more than 10M uuids/sec');
      }

      _lastMSecs = msecs;
      _lastNSecs = nsecs;
      _clockseq = clockseq;

      // Per 4.1.4 - Convert from unix epoch to Gregorian epoch
      msecs += 12219292800000;

      // `time_low`
      var tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
      b[i++] = tl >>> 24 & 0xff;
      b[i++] = tl >>> 16 & 0xff;
      b[i++] = tl >>> 8 & 0xff;
      b[i++] = tl & 0xff;

      // `time_mid`
      var tmh = (msecs / 0x100000000 * 10000) & 0xfffffff;
      b[i++] = tmh >>> 8 & 0xff;
      b[i++] = tmh & 0xff;

      // `time_high_and_version`
      b[i++] = tmh >>> 24 & 0xf | 0x10; // include version
      b[i++] = tmh >>> 16 & 0xff;

      // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)
      b[i++] = clockseq >>> 8 | 0x80;

      // `clock_seq_low`
      b[i++] = clockseq & 0xff;

      // `node`
      for (var n = 0; n < 6; ++n) {
        b[i + n] = node[n];
      }

      return buf ? buf : bytesToUuid_1(b);
    }

    var v1_1 = v1;

    function v4(options, buf, offset) {
      var i = buf && offset || 0;

      if (typeof(options) == 'string') {
        buf = options === 'binary' ? new Array(16) : null;
        options = null;
      }
      options = options || {};

      var rnds = options.random || (options.rng || rng)();

      // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
      rnds[6] = (rnds[6] & 0x0f) | 0x40;
      rnds[8] = (rnds[8] & 0x3f) | 0x80;

      // Copy bytes to buffer, if provided
      if (buf) {
        for (var ii = 0; ii < 16; ++ii) {
          buf[i + ii] = rnds[ii];
        }
      }

      return buf || bytesToUuid_1(rnds);
    }

    var v4_1 = v4;

    var uuid = v4_1;
    uuid.v1 = v1_1;
    uuid.v4 = v4_1;

    var uuid_1 = uuid;
    var uuid_2 = uuid_1.v1;

    // TODO: Would be great if this works.
    // K extends undefined
    //     ? { name: string; Class: ActorCons<T> }
    //     : { name: string; Class: ActorCons<T, K>; paramOptions: K };
    var ActorSystem = /** @class */ (function () {
        function ActorSystem(name) {
            var _this = this;
            this.actorSystemRegistry = {};
            this.createActor = function (options) {
                var name = options.name, actorClass = options.actorClass;
                _this.log("Creating an actor with name: " + name + " and type: " + actorClass.name);
                var address = name; // TODO: should have a proper mechanism to generate address
                var fullAddress = {
                    actorSystemName: _this.name,
                    localAddress: address
                };
                _this.actorRegistry[address] = new actorClass(name, fullAddress, _this, options.paramOptions, options.strategies);
                return _this.ref(fullAddress);
            };
            this.removeActor = function (refOrAddress) {
                var address;
                if (refOrAddress instanceof ActorRef) {
                    address = refOrAddress.address;
                }
                else {
                    address = refOrAddress;
                }
                if (address.actorSystemName !== _this.name) {
                    throw new Error("Cannot remove actor that does not belong to this actor system");
                }
                var actor = _this.actorRegistry[address.localAddress];
                if (actor) {
                    delete _this.actorRegistry[address.localAddress];
                }
                else {
                    throw new Error("Unable to remove actor. The specified actor not found");
                }
            };
            this.ref = function (address) {
                return new ActorRef(address, _this);
            };
            this.findActor = function (address) {
                if (address.actorSystemName !== _this.name) {
                    _this.log("This address contains reference to other actor system, you won't find it in this actor system");
                    return null;
                }
                var actor = _this.actorRegistry[address.localAddress];
                if (actor) {
                    return new ActorRef(address, _this);
                }
                else {
                    return null;
                }
            };
            this.sendMessage = function (target, type, senderAddress) {
                var payload = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    payload[_i - 3] = arguments[_i];
                }
                _this.sendMessageAndWait.apply(_this, [target, type, senderAddress].concat(payload)).then(function () {
                    /* do nothing */
                }, function (error) {
                    _this.log("Catch an error when executing message with type " + type, "Target", target, "Sender", senderAddress, "Payload", payload, "Error", error);
                });
            };
            this.sendMessageAndWait = function (target, type, senderAddress) {
                var payload = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    payload[_i - 3] = arguments[_i];
                }
                _this.log("Received a request to send a message with type: " + type, "Target", target, "Sender", senderAddress, "Payload", payload);
                var address;
                if (target instanceof ActorRef) {
                    address = target.address;
                }
                else {
                    address = target;
                }
                if (_this.isLocalAddress(address)) {
                    var actor = _this.actorRegistry[address.localAddress];
                    if (actor) {
                        _this.log("Found the actor. Sending the message");
                        return actor.pushToMailbox.apply(actor, [type, senderAddress].concat(payload));
                    }
                    else {
                        _this.log("Unable to find the actor. It might have died");
                        return Promise.reject("Actor not found");
                    }
                }
                else {
                    var actorSystemEmitter_1 = _this.actorSystemRegistry[address.actorSystemName];
                    if (actorSystemEmitter_1) {
                        return new Promise(function (resolve, reject) {
                            _this.log("Found the actor system. Sending the message");
                            actorSystemEmitter_1.emit("message", {
                                mode: "ask",
                                targetAddress: address,
                                senderAddress: senderAddress,
                                type: type,
                                payload: payload
                            }, function (message) { return resolve(message); });
                        });
                    }
                    else {
                        _this.log("Cannot find the targeted actor system");
                        return Promise.reject("ActorSystem not found");
                    }
                }
            };
            this.name = name || uuid_2();
            this.actorRegistry = {};
        }
        ActorSystem.prototype.register = function (emitter) {
            this.listenTo(emitter);
            this.log("Send a handshake message");
            emitter.emit("message", { mode: "handshake", address: this.name });
        };
        ActorSystem.prototype.unregister = function (actorSystemAddress) {
            delete this.actorSystemRegistry[actorSystemAddress];
        };
        ActorSystem.prototype.listenTo = function (emitter) {
            var _this = this;
            var actorSystemAddress = undefined;
            emitter.on("disconnect", function () {
                _this.log("Removing listener");
                if (actorSystemAddress) {
                    _this.unregister(actorSystemAddress);
                }
            });
            emitter.on("message", function (interActorSystemMessage, cb) {
                if (interActorSystemMessage.mode === "handshake") {
                    _this.log("Received a handshake message", interActorSystemMessage);
                    actorSystemAddress = interActorSystemMessage.address;
                    _this.actorSystemRegistry[actorSystemAddress] = emitter;
                }
                else {
                    _this.log("Received a message from across the system boundary", interActorSystemMessage);
                    var actorRef = _this.findActor(interActorSystemMessage.targetAddress);
                    _this.log("The destination address is", interActorSystemMessage.targetAddress);
                    if (actorRef) {
                        var mode = interActorSystemMessage.mode, type_1 = interActorSystemMessage.type, payload_1 = interActorSystemMessage.payload, senderAddress_1 = interActorSystemMessage.senderAddress;
                        if (mode === "send") {
                            _this.log("Sending the message to the appropriate actor. Type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                            _this.sendMessageAndWait.apply(_this, [actorRef, type_1, senderAddress_1].concat(payload_1));
                        }
                        else {
                            _this.log("Sending the question to the appropriate actor. Type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                            _this.sendMessageAndWait.apply(_this, [actorRef, type_1, senderAddress_1].concat(payload_1)).then(function (message) {
                                _this.log("Received an answer, sending the answer \"" + message + "\" for the question with type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                                cb(message);
                            });
                        }
                    }
                    else {
                        _this.log("Unable to find the recipient of the message");
                    }
                }
            });
        };
        ActorSystem.prototype.isLocalAddress = function (address) {
            return address.actorSystemName === this.name;
        };
        ActorSystem.prototype.log = function () {
            var message = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                message[_i] = arguments[_i];
            }
            if ((process.env && process.env.ACTRIX_DEBUG) ||
                (typeof window !== "undefined" && window.ACTRIX_DEBUG)) {
                console.log.apply(console, ["[" + this.name + "]"].concat(message));
            }
        };
        return ActorSystem;
    }());

    function promisify(generator) {
        // Implementation based on https://github.com/tj/co/blob/master/index.js
        return function () {
            var args = arguments;
            var ctx = this;
            var gen = generator.apply(ctx, args);
            var rejector;
            var currentPromise = undefined;
            var res = new Promise(function (resolve, reject) {
                rejector = reject;
                function onFulfilled(res) {
                    currentPromise = undefined;
                    var result;
                    try {
                        result = gen.next(res);
                    }
                    catch (e) {
                        reject(e);
                        return;
                    }
                    // noinspection JSIgnoredPromiseFromCall
                    next(result);
                }
                function onRejected(err) {
                    currentPromise = undefined;
                    var result;
                    try {
                        result = gen.throw(err);
                    }
                    catch (e) {
                        reject(e);
                        return;
                    }
                    // noinspection JSIgnoredPromiseFromCall
                    next(result);
                }
                function next(ret) {
                    if (ret && typeof ret.then === "function") {
                        // an async iterator
                        ret.then(next, reject);
                        return;
                    }
                    if (ret.done) {
                        resolve(ret.value);
                        return;
                    }
                    currentPromise = Promise.resolve(ret.value);
                    return currentPromise.then(onFulfilled, onRejected);
                }
                onFulfilled(undefined); // kick off the process
            });
            res.cancel = function () {
                try {
                    if (currentPromise) {
                        cancelPromise(currentPromise);
                    }
                    var res_1 = gen.return();
                    // eat anything that promise would do, it's cancelled!
                    var yieldedPromise = Promise.resolve(res_1.value);
                    yieldedPromise.then(function () {
                        /* nothing */
                    }, function () {
                        /* nothing */
                    });
                    cancelPromise(yieldedPromise); // maybe it can be cancelled :)
                    rejector(new Error("PROMISE_CANCELLED"));
                }
                catch (e) {
                    rejector(e); // there could be a throwing finally block
                }
            };
            return res;
        };
    }
    function cancelPromise(promise) {
        if (typeof promise.cancel === "function") {
            promise.cancel();
        }
    }

    var TestActorSystem = /** @class */ (function (_super) {
        __extends(TestActorSystem, _super);
        function TestActorSystem() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TestActorSystem.prototype.getRealActor = function (localAddress) {
            return this.actorRegistry[localAddress];
        };
        return TestActorSystem;
    }(ActorSystem));

    // Import here Polyfills if needed. Recommended core-js (npm i -D core-js)

    exports.ActorRef = ActorRef;
    exports.Actor = Actor;
    exports.ActorSystem = ActorSystem;
    exports.promisify = promisify;
    exports.TestActorSystem = TestActorSystem;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
