"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
function createProxy(actorSystem, targetAddressorActorRef, sender, ask) {
    if (ask === void 0) { ask = false; }
    return new Proxy({}, {
        get: function (target, prop, receiver) {
            return function () {
                var payload = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    payload[_i] = arguments[_i];
                }
                return ask
                    ? actorSystem.sendMessageAndWait.apply(actorSystem, [targetAddressorActorRef,
                        prop,
                        sender || null].concat(payload)) : actorSystem.sendMessage.apply(actorSystem, [targetAddressorActorRef,
                    prop,
                    sender || null].concat(payload));
            };
        }
    });
}
var ActorRef = /** @class */ (function () {
    function ActorRef(address, actorSystem) {
        this.address = address;
        this.actorSystem = actorSystem;
    }
    ActorRef.prototype.send = function (sender) {
        return createProxy(this.actorSystem, this.address, sender);
    };
    ActorRef.prototype.ask = function (sender) {
        return createProxy(this.actorSystem, this.address, sender, true);
    };
    return ActorRef;
}());
exports.ActorRef = ActorRef;
var Actor = /** @class */ (function () {
    function Actor(name, address, actorSystem, options, strategies) {
        var _this = this;
        this.address = address;
        this.actorSystem = actorSystem;
        this.strategies = strategies;
        this.mailBox = [];
        this.context = {
            senderAddress: null,
            senderRef: null
        };
        this.pushToMailbox = function (type, senderAddress) {
            var payload = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                payload[_i - 2] = arguments[_i];
            }
            _this.log("A new message with type " + type + " and payload " + payload + " from sender " + senderAddress + " is received");
            if (_this.strategies && _this.strategies.includes("IgnoreOlderMessageWithTheSameType")) {
                if (_this.currentlyProcessedMessage && _this.currentlyProcessedMessage.type === type) {
                    _this.cancelCurrentExecution();
                }
            }
            try {
                _this.onNewMessage.apply(_this, [type, senderAddress].concat(payload));
            }
            catch (error) {
                _this.log("Caught an exception on the implementation of 'onNewMessage' on actor " + _this.name, error);
            }
            var promise = new Promise(function (resolve, reject) {
                _this.mailBox.push({
                    type: type,
                    payload: payload,
                    senderAddress: senderAddress,
                    callback: function (error, result) {
                        if (error) {
                            reject(error);
                        }
                        else {
                            resolve(result);
                        }
                    }
                });
            });
            _this.scheduleNextTick();
            return promise;
        };
        this.ref = function (address) {
            return _this.actorSystem.ref(address);
        };
        this.onNewMessage = function (type, senderAddress) {
            var payload = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                payload[_i - 2] = arguments[_i];
            }
            // should be overridden by implementator (when necessary)
        };
        this.cancelCurrentExecution = function () {
            _this.log("Cancel current execution");
            if (_this.currentPromise && typeof _this.currentPromise.cancel === "function") {
                _this.currentPromise.cancel();
            }
        };
        this.scheduleNextTick = function () {
            if (!_this.timerId) {
                _this.timerId = setTimeout(_this.executeTick);
            }
        };
        this.executeTick = function () { return __awaiter(_this, void 0, void 0, function () {
            var mail, success, result, _a, type, payload, senderAddress, callback, error_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        mail = this.mailBox.shift();
                        success = false;
                        _a = mail, type = _a.type, payload = _a.payload, senderAddress = _a.senderAddress, callback = _a.callback;
                        this.context = {
                            senderAddress: senderAddress,
                            senderRef: senderAddress ? this.ref(senderAddress) : null
                        };
                        if (this.strategies && this.strategies.includes("IgnoreOlderMessageWithTheSameType")) {
                            if (this.mailBox.find(function (message) { return message.type === type; })) {
                                this.log("Ignoring message with type " + type + " because there are more messages with the same type in the queue.");
                                result = "There are new messages with the same type. This message is ignored";
                            }
                        }
                        this.currentlyProcessedMessage = mail;
                        if (!!result) return [3 /*break*/, 4];
                        this.currentPromise = this.handleMessage.apply(this, [type].concat(payload));
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.currentPromise];
                    case 2:
                        result = _b.sent();
                        this.log("Output of the handled message", result);
                        success = true;
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _b.sent();
                        this.log("Caught an exception when handling message", error_1);
                        result = error_1;
                        return [3 /*break*/, 4];
                    case 4:
                        this.currentlyProcessedMessage = undefined;
                        this.currentPromise = undefined;
                        callback(success ? undefined : result, success ? result : undefined);
                        if (this.timerId) {
                            clearTimeout(this.timerId);
                            this.timerId = null;
                        }
                        if (this.mailBox.length) {
                            this.scheduleNextTick();
                        }
                        return [2 /*return*/];
                }
            });
        }); };
        this.name = name;
        this.timerId = null;
        setTimeout(function () {
            _this.init(options);
        });
    }
    Actor.prototype.sendTo = function (targetRef) {
        return createProxy(this.actorSystem, targetRef, this.address);
    };
    Actor.prototype.askTo = function (targetRef) {
        return createProxy(this.actorSystem, targetRef, this.address, true);
    };
    // For some reason the typings is not working properly
    Actor.prototype.sendToSelf = function () {
        // return this.at<ValidActorMethodProps<this>>(this.address);
        return this.sendTo(this.address); // TODO: introduce generic for actor
    };
    Actor.prototype.init = function (options) {
        // can be implemented by the concrete actor
    };
    Actor.prototype.log = function () {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        if ((process.env && process.env.ACTRIX_DEBUG) ||
            (typeof window !== "undefined" && window.ACTRIX_DEBUG)) {
            console.log.apply(console, ["[" + this.name + "]"].concat(message));
        }
    };
    // TODO: K extends keyof this is not actually the proper solution,
    // good for now though since it doesn't affect end-user
    Actor.prototype.handleMessage = function (type) {
        var payload = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            payload[_i - 1] = arguments[_i];
        }
        this.log("Handling message of type " + type + " and payload " + payload);
        return (_a = this)[type].apply(_a, payload);
        var _a;
    };
    return Actor;
}());
exports.Actor = Actor;
//# sourceMappingURL=Actor.js.map