"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Actor_1 = require("./Actor");
var uuid = require("uuid");
// TODO: Would be great if this works.
// K extends undefined
//     ? { name: string; Class: ActorCons<T> }
//     : { name: string; Class: ActorCons<T, K>; paramOptions: K };
var ActorSystem = /** @class */ (function () {
    function ActorSystem(name) {
        var _this = this;
        this.actorSystemRegistry = {};
        this.createActor = function (options) {
            var name = options.name, actorClass = options.actorClass;
            _this.log("Creating an actor with name: " + name + " and type: " + actorClass.name);
            var address = name; // TODO: should have a proper mechanism to generate address
            var fullAddress = {
                actorSystemName: _this.name,
                localAddress: address
            };
            _this.actorRegistry[address] = new actorClass(name, fullAddress, _this, options.paramOptions, options.strategies);
            return _this.ref(fullAddress);
        };
        this.removeActor = function (refOrAddress) {
            var address;
            if (refOrAddress instanceof Actor_1.ActorRef) {
                address = refOrAddress.address;
            }
            else {
                address = refOrAddress;
            }
            if (address.actorSystemName !== _this.name) {
                throw new Error("Cannot remove actor that does not belong to this actor system");
            }
            var actor = _this.actorRegistry[address.localAddress];
            if (actor) {
                delete _this.actorRegistry[address.localAddress];
            }
            else {
                throw new Error("Unable to remove actor. The specified actor not found");
            }
        };
        this.ref = function (address) {
            return new Actor_1.ActorRef(address, _this);
        };
        this.findActor = function (address) {
            if (address.actorSystemName !== _this.name) {
                _this.log("This address contains reference to other actor system, you won't find it in this actor system");
                return null;
            }
            var actor = _this.actorRegistry[address.localAddress];
            if (actor) {
                return new Actor_1.ActorRef(address, _this);
            }
            else {
                return null;
            }
        };
        this.sendMessage = function (target, type, senderAddress) {
            var payload = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                payload[_i - 3] = arguments[_i];
            }
            _this.sendMessageAndWait.apply(_this, [target, type, senderAddress].concat(payload)).then(function () {
                /* do nothing */
            }, function (error) {
                _this.log("Catch an error when executing message with type " + type, "Target", target, "Sender", senderAddress, "Payload", payload, "Error", error);
            });
        };
        this.sendMessageAndWait = function (target, type, senderAddress) {
            var payload = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                payload[_i - 3] = arguments[_i];
            }
            _this.log("Received a request to send a message with type: " + type, "Target", target, "Sender", senderAddress, "Payload", payload);
            var address;
            if (target instanceof Actor_1.ActorRef) {
                address = target.address;
            }
            else {
                address = target;
            }
            if (_this.isLocalAddress(address)) {
                var actor = _this.actorRegistry[address.localAddress];
                if (actor) {
                    _this.log("Found the actor. Sending the message");
                    return actor.pushToMailbox.apply(actor, [type, senderAddress].concat(payload));
                }
                else {
                    _this.log("Unable to find the actor. It might have died");
                    return Promise.reject("Actor not found");
                }
            }
            else {
                var actorSystemEmitter_1 = _this.actorSystemRegistry[address.actorSystemName];
                if (actorSystemEmitter_1) {
                    return new Promise(function (resolve, reject) {
                        _this.log("Found the actor system. Sending the message");
                        actorSystemEmitter_1.emit("message", {
                            mode: "ask",
                            targetAddress: address,
                            senderAddress: senderAddress,
                            type: type,
                            payload: payload
                        }, function (message) { return resolve(message); });
                    });
                }
                else {
                    _this.log("Cannot find the targeted actor system");
                    return Promise.reject("ActorSystem not found");
                }
            }
        };
        this.name = name || uuid.v1();
        this.actorRegistry = {};
    }
    ActorSystem.prototype.register = function (emitter) {
        this.listenTo(emitter);
        this.log("Send a handshake message");
        emitter.emit("message", { mode: "handshake", address: this.name });
    };
    ActorSystem.prototype.unregister = function (actorSystemAddress) {
        delete this.actorSystemRegistry[actorSystemAddress];
    };
    ActorSystem.prototype.listenTo = function (emitter) {
        var _this = this;
        var actorSystemAddress = undefined;
        emitter.on("disconnect", function () {
            _this.log("Removing listener");
            if (actorSystemAddress) {
                _this.unregister(actorSystemAddress);
            }
        });
        emitter.on("message", function (interActorSystemMessage, cb) {
            if (interActorSystemMessage.mode === "handshake") {
                _this.log("Received a handshake message", interActorSystemMessage);
                actorSystemAddress = interActorSystemMessage.address;
                _this.actorSystemRegistry[actorSystemAddress] = emitter;
            }
            else {
                _this.log("Received a message from across the system boundary", interActorSystemMessage);
                var actorRef = _this.findActor(interActorSystemMessage.targetAddress);
                _this.log("The destination address is", interActorSystemMessage.targetAddress);
                if (actorRef) {
                    var mode = interActorSystemMessage.mode, type_1 = interActorSystemMessage.type, payload_1 = interActorSystemMessage.payload, senderAddress_1 = interActorSystemMessage.senderAddress;
                    if (mode === "send") {
                        _this.log("Sending the message to the appropriate actor. Type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                        _this.sendMessageAndWait.apply(_this, [actorRef, type_1, senderAddress_1].concat(payload_1));
                    }
                    else {
                        _this.log("Sending the question to the appropriate actor. Type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                        _this.sendMessageAndWait.apply(_this, [actorRef, type_1, senderAddress_1].concat(payload_1)).then(function (message) {
                            _this.log("Received an answer, sending the answer \"" + message + "\" for the question with type: " + type_1 + ", sender: " + senderAddress_1 + ", and payload:", payload_1);
                            cb(message);
                        });
                    }
                }
                else {
                    _this.log("Unable to find the recipient of the message");
                }
            }
        });
    };
    ActorSystem.prototype.isLocalAddress = function (address) {
        return address.actorSystemName === this.name;
    };
    ActorSystem.prototype.log = function () {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        if ((process.env && process.env.ACTRIX_DEBUG) ||
            (typeof window !== "undefined" && window.ACTRIX_DEBUG)) {
            console.log.apply(console, ["[" + this.name + "]"].concat(message));
        }
    };
    return ActorSystem;
}());
exports.ActorSystem = ActorSystem;
//# sourceMappingURL=ActorSystem.js.map