import { ActorSystem } from "./ActorSystem";
import { Address, Strategy } from "./interfaces";
export declare type MailBoxMessage<T> = {
    type: ValidActorMethodPropNames<T>;
    payload: PayloadPropNames<T>[];
    senderAddress: Address | null;
    callback: (error?: any, result?: any) => void;
};
export declare type ActorSendAPI<T> = {
    [K in ValidActorMethodPropNames<T>]: T[K] extends () => any ? () => void : T[K] extends (arg1: infer A1) => any ? (arg: A1) => void : T[K] extends (arg1: infer A1, arg2: infer A2) => any ? (arg1: A1, arg2: A2) => void : T[K] extends (arg1: infer A1, arg2: infer A2, arg3: infer A3) => any ? (arg1: A1, arg2: A2, arg3: A3) => void : T[K] extends (arg1: infer A1, arg2: infer A2, arg3: infer A3, arg4: infer A4) => any ? (arg1: A1, arg2: A2, arg3: A3, arg4: A4) => void : T[K] extends (arg1: infer A1, arg2: infer A2, arg3: infer A3, arg4: infer A4, arg5: infer A5) => any ? (arg1: A1, arg2: A2, arg3: A3, arg4: A4, arg5: A5) => void : never;
};
export declare type ActorAskAPI<T> = Pick<T, ValidActorMethodPropNames<T>>;
export declare type ValidActorMethodPropNames<T> = {
    [K in Exclude<keyof T, keyof Actor>]: T[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never;
}[Exclude<keyof T, keyof Actor>];
export declare type PayloadPropNames<T> = {
    [K in Exclude<keyof T, keyof Actor>]: T[K] extends (_: infer S) => Promise<any> ? S : never;
}[Exclude<keyof T, keyof Actor>];
export declare type ActorCons<T extends Actor<K>, K = undefined> = new (name: string, address: Address, actorSystem: ActorSystem, options?: K, strategies?: Strategy[]) => T;
export declare class ActorRef<T> {
    address: Address;
    private actorSystem;
    constructor(address: Address, actorSystem: ActorSystem);
    send(sender?: Address): ActorSendAPI<T>;
    ask(sender?: Address): Pick<T, { [K in Exclude<keyof T, "pushToMailbox">]: T[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof T, "pushToMailbox">]>;
}
export declare abstract class Actor<InitParam = undefined> {
    protected address: Address;
    protected actorSystem: ActorSystem;
    protected strategies: "IgnoreOlderMessageWithTheSameType"[] | undefined;
    protected name: string;
    protected mailBox: MailBoxMessage<this>[];
    protected currentlyProcessedMessage: MailBoxMessage<this> | undefined;
    protected context: {
        senderAddress: Address | null;
        senderRef: ActorRef<any> | null;
    };
    private timerId;
    private currentPromise;
    constructor(name: string, address: Address, actorSystem: ActorSystem, options?: InitParam, strategies?: "IgnoreOlderMessageWithTheSameType"[] | undefined);
    pushToMailbox: <K extends { [K in Exclude<keyof this, "pushToMailbox">]: this[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof this, "pushToMailbox">], L extends { [K in Exclude<keyof this, "pushToMailbox">]: this[K] extends (_: infer S) => Promise<any> ? S : never; }[Exclude<keyof this, "pushToMailbox">]>(type: K, senderAddress: Address | null, ...payload: L[]) => Promise<any>;
    protected ref: <T>(address: Address) => ActorRef<T>;
    protected sendTo<A>(targetRef: ActorRef<A> | Address): ActorSendAPI<A>;
    protected askTo<A>(targetRef: ActorRef<A> | Address): Pick<A, { [K in Exclude<keyof A, "pushToMailbox">]: A[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof A, "pushToMailbox">]>;
    protected sendToSelf(): ActorSendAPI<any>;
    protected onNewMessage: <K extends { [K in Exclude<keyof this, "pushToMailbox">]: this[K] extends (...args: any[]) => infer R ? R extends Promise<any> ? K : never : never; }[Exclude<keyof this, "pushToMailbox">], L extends { [K in Exclude<keyof this, "pushToMailbox">]: this[K] extends (_: infer S) => Promise<any> ? S : never; }[Exclude<keyof this, "pushToMailbox">]>(type: K, senderAddress: Address | null, ...payload: L[]) => void;
    protected init(options?: InitParam): void;
    protected log(...message: any[]): void;
    protected cancelCurrentExecution: () => void;
    private handleMessage<K>(type, ...payload);
    private scheduleNextTick;
    private executeTick;
}
